package de.swe2.artikelverwaltung.service;

import static de.swe2.util.AbstractDao.QueryParameter.with;

import java.io.Serializable;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import javax.annotation.PreDestroy;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.validation.ConstraintViolation;
import javax.validation.Validator;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.swe2.artikelverwaltung.domain.Artikel;
import de.swe2.util.IdGroup;
import de.swe2.util.NotFoundException;
import de.swe2.util.ValidationService;


@Stateless
public class Artikelverwaltung implements Serializable {
	private static final long serialVersionUID = 7954453539970303611L;
	private static final Logger LOGGER = LoggerFactory.getLogger(Artikelverwaltung.class);
	
	@SuppressWarnings("unused")
	@PersistenceContext
	private transient EntityManager em;
	
	@EJB
	private ArtikelverwaltungDao dao;
	
	@EJB
	private ValidationService validationService;
	
	@SuppressWarnings("unused")
	@PreDestroy
	private void preDestroy() {
		LOGGER.debug("Session Bean {} wird geloescht", Artikelverwaltung.class.getSimpleName());
	}
	
	/**
	 */
	public List<Artikel> findVerfuegbareArtikel() throws NotFoundException {
		final List<Artikel> artikel = dao.find(Artikel.FIND_VERFUEGBARE_ARTIKEL, Artikel.class);
		return artikel;
	}

	
	/**
	 */
	public Artikel findArtikelById(Long id, Locale locale) throws InvalidArtikelIdException, NotFoundException {
		validateArtikelId(id, locale);
		
		final Artikel artikel = dao.find(Artikel.class, id);
		return artikel;
	}
	
	private void validateArtikelId(Long id, Locale locale) throws InvalidArtikelIdException {
		LOGGER.debug("BEGINN validateArtikelId: id={}", id);
		final Validator validator = validationService.getValidator(locale);
		Set<ConstraintViolation<Artikel>> violations = validator.validateValue(Artikel.class, "id", id, IdGroup.class);	
		
		if (!violations.isEmpty()) {
			throw new InvalidArtikelIdException(id, violations);
		}		
		LOGGER.debug("ENDE validateArtikelId");
	}
	
	/**
	 */
	public List<Artikel> findArtikelByBezeichnung(String bezeichnung) throws NotFoundException {
		if (bezeichnung == null || bezeichnung.isEmpty()) {
			final List<Artikel> artikelListe = findVerfuegbareArtikel();
			return artikelListe;
		}
		
		final List<Artikel> artikelListe = dao.find(Artikel.FIND_ARTIKEL_BY_BEZ,
				                                    with(Artikel.PARAM_BEZEICHNUNG, "%" + bezeichnung + "%").parameters(),
													Artikel.class);
		
		return artikelListe;
	}
	
	/**
	 */
	public List<Artikel> findArtikelByIds(List<Long> ids, Locale locale) throws InvalidArtikelIdException, NotFoundException {
		for (Long id : ids) {
			validateArtikelId(id, locale);
		}
		
		final List<Artikel> artikel = dao.findArtikelByIds(ids);
		return artikel;
	}
}
