package de.swe2.artikelverwaltung.service;

import static javax.ejb.TransactionAttributeType.MANDATORY;

import java.util.Collections;
import java.util.List;

import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Path;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import de.swe2.artikelverwaltung.domain.Artikel;
import de.swe2.util.AbstractDao;
import de.swe2.util.NotFoundException;

@Stateless
@TransactionAttribute(MANDATORY)
public class ArtikelverwaltungDao extends AbstractDao {
	private static final long serialVersionUID = -7695451728933515022L;

	/**
	 * SELECT a
	 * FROM   Artikel a
	 * WHERE  a.id = ? OR a.id = ? OR ...
	 */
	public List<Artikel> findArtikelByIds(List<Long> ids) throws NotFoundException {
		if (ids == null || ids.isEmpty()) {
			return Collections.emptyList();
		}
		
		final CriteriaBuilder builder = em.getCriteriaBuilder();
		final CriteriaQuery<Artikel> criteriaQuery = builder.createQuery(Artikel.class);
		final Root<Artikel> a = criteriaQuery.from(Artikel.class);

		final Path<Long> idPath = a.get("id");
		//final Path<String> idPath = a.get(Artikel_.id);   // Metamodel-Klassen funktionieren nicht mit Eclipse
		
		Predicate pred = null;
		if (ids.size() == 1) {
			// Genau 1 id: kein OR notwendig
			pred = builder.equal(idPath, ids.get(0));
		}
		else {
			// Mind. 2x id, durch OR verknuepft
			final Predicate[] equals = new Predicate[ids.size()];
			int i = 0;
			for (Long id : ids) {
				equals[i++] = builder.equal(idPath, id);
			}
			
			pred = builder.or(equals);
		}
		
		criteriaQuery.where(pred);
		
		final TypedQuery<Artikel> query = em.createQuery(criteriaQuery);

		final List<Artikel> artikel = query.getResultList();
		if (artikel.isEmpty()) {
			throw new NotFoundException("Keine Treffer fuer die IDs: " + ids, Artikel.class);
		}

		return artikel;
	}
}
