package de.swe2.bestellverwaltung.domain;

import static de.swe2.util.EjbConstants.ERSTE_VERSION;
import static de.swe2.util.EjbConstants.KEINE_ID;
import static javax.xml.bind.annotation.XmlAccessType.FIELD;

import java.net.URI;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.Version;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import de.swe2.artikelverwaltung.domain.Artikel;


@Entity
@Table(name = "bestellposition")
@NamedQueries({
    @NamedQuery(name  = Bestellposition.FIND_LADENHUETER,
   	            query = "SELECT a"
   	            	    + " FROM   Artikel a"
   	            	    + " WHERE  a NOT IN (SELECT bp.artikel FROM Bestellposition bp)")
})
@XmlRootElement
@XmlAccessorType(FIELD)
public class Bestellposition implements java.io.Serializable {
	private static final long serialVersionUID = 1031749849939138054L;
	
	private static final String PREFIX = "Bestellposition.";
	public static final String FIND_LADENHUETER = PREFIX + "findLadenhueter";
	private static final int ANZAHL_MIN = 1;

	@Id
	@GeneratedValue(generator = "bestellposition_sequence_name")
	@SequenceGenerator(name = "bestellposition_sequence_name", sequenceName = "bestellposition_bp_id_seq", allocationSize = 1)
	@Column(name = "bp_id", nullable = false)
	@XmlAttribute(name = "id", required = true)
	private Long id = KEINE_ID;

	@Version
	@XmlTransient
	private int version = ERSTE_VERSION;
	
	@Column(nullable = false)
	@Min(value = ANZAHL_MIN, message = "{bestellverwaltung.bestellposition.anzahl.min}")
	private short anzahl;
	
	@ManyToOne(optional = false)
	@JoinColumn(name = "artikel_fk")
	@NotNull(message = "{bestellverwaltung.bestellposition.artikel.notNull}")
	@XmlTransient
	private Artikel artikel;
	
	@Transient
	@XmlElement(name = "artikel")
	private URI artikelKey;
	
	public Bestellposition() {
		super();
	}
	
	public Bestellposition(Artikel artikel) {
		super();
		this.artikel = artikel;
		this.anzahl = 1;
	}
	
	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public int getVersion() {
		return version;
	}

	public void setVersion(int version) {
		this.version = version;
	}

	public short getAnzahl() {
		return anzahl;
	}
	public void setAnzahl(short anzahl) {
		this.anzahl = anzahl;
	}
	
	public Artikel getArtikel() {
		return artikel;
	}
	public void setArtikel(Artikel artikel) {
		this.artikel = artikel;
	}

	public URI getArtikelKey() {
		return artikelKey;
	}
	
	public void setArtikelKey(URI artikelKey) {
		this.artikelKey = artikelKey;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + anzahl;
		result = prime * result + ((artikel == null) ? 0 : artikel.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final Bestellposition other = (Bestellposition) obj;
		if (anzahl != other.anzahl) {
			return false;
		}
		if (artikel == null) {
			if (other.artikel != null) {
				return false;
			}
		}
		else if (!artikel.equals(other.artikel)) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {
		if (artikel == null) {
			return "Bestellposition [id=" + id + ", version=" + version + ", artikelKey=" + artikelKey + ", anzahl=" + anzahl + "]";
		}

		return "Bestellposition [id=" + id + ", version=" + version + ", artikel.id=" + artikel.getId() + ", anzahl=" + anzahl + "]";
	}

}
