package de.swe2.bestellverwaltung.domain;

import static de.swe2.util.EjbConstants.ERSTE_VERSION;
import static de.swe2.util.EjbConstants.KEINE_ID;
import static javax.persistence.CascadeType.PERSIST;
import static javax.persistence.CascadeType.REMOVE;
import static javax.persistence.FetchType.EAGER;
import static javax.persistence.TemporalType.TIMESTAMP;
import static javax.xml.bind.annotation.XmlAccessType.FIELD;

import java.net.URI;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.OrderColumn;
import javax.persistence.PostPersist;
import javax.persistence.PostUpdate;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.Transient;
import javax.persistence.Version;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import org.hibernate.validator.constraints.NotEmpty;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.swe2.kundenverwaltung.domain.AbstractKunde;


@Entity
@Table(name = "bestellung")
@NamedQueries({
	@NamedQuery(name  = Bestellung.FIND_BESTELLUNGEN_BY_KUNDEID,
                query = "SELECT b"
				        + " FROM   Bestellung b"
			            + " WHERE  b.kunde.id = :" + Bestellung.PARAM_KUNDEID),
	@NamedQuery(name  = Bestellung.FIND_BESTELLUNGEN_BY_KUNDEID_FETCH_LIEFERUNGEN,
                query = "SELECT DISTINCT b"
                        + " FROM   Bestellung b LEFT JOIN FETCH b.lieferungen"
	                    + " WHERE  b.kunde.id = :" + Bestellung.PARAM_KUNDEID),
	@NamedQuery(name  = Bestellung.FIND_KUNDE_BY_ID,
	            query = "SELECT DISTINCT b.kunde"
	                    + " FROM   Bestellung b"
			            + " WHERE  b.id = :" + Bestellung.PARAM_ID),
	@NamedQuery(name  = Bestellung.FIND_BESTELLUNG_BY_ID_FETCH_LIEFERUNGEN,
                query = "SELECT DISTINCT b"
                        + " FROM   Bestellung b LEFT JOIN FETCH b.lieferungen"
   			            + " WHERE  b.id = :" + Bestellung.PARAM_ID),
	@NamedQuery(name  = Bestellung.FIND_ANZ_BESTELLUNGEN_BY_PLZ_V1,
                query = "SELECT COUNT(b)"
                        + " FROM   Bestellung b"
                        + " WHERE  b.kunde.adresse.plz BETWEEN :" + Bestellung.PARAM_PLZ_MIN
                        +        " AND :" + Bestellung.PARAM_PLZ_MAX),
   	@NamedQuery(name  = Bestellung.FIND_ANZ_BESTELLUNGEN_BY_PLZ_V2,
                query = "SELECT COUNT(b)"
                        + " FROM   AbstractKunde k JOIN k.bestellungen b"
                        + " WHERE  k.adresse.plz BETWEEN :" + Bestellung.PARAM_PLZ_MIN
                        +        " AND :" + Bestellung.PARAM_PLZ_MAX),
    @NamedQuery(name  = Bestellung.FIND_PLZ_ANZAHL_BESTELLUNGEN,
   	            query = "SELECT   NEW de.swe2.bestellverwaltung.domain.PlzAnzahlBestellungen(b.kunde.adresse.plz, COUNT(b))"
   	            	    + " FROM     Bestellung b"
   	            	    + " GROUP BY b.kunde.adresse.plz")
})
@XmlRootElement
@XmlAccessorType(FIELD)
public class Bestellung implements java.io.Serializable {
	private static final long serialVersionUID = 7560752199018702446L;
	protected static final Logger LOGGER = LoggerFactory.getLogger(Bestellung.class);
	
	private static final String PREFIX = "Bestellung.";
	public static final String FIND_BESTELLUNGEN_BY_KUNDEID = PREFIX + "findBestellungenByKundeId";
	public static final String FIND_BESTELLUNGEN_BY_KUNDEID_FETCH_LIEFERUNGEN = PREFIX + "findBestellungenByKundeIdFetchLieferungen";
	public static final String FIND_KUNDE_BY_ID = PREFIX + "findKundeById";
	public static final String FIND_BESTELLUNG_BY_ID_FETCH_LIEFERUNGEN = PREFIX + "findBestellungenByIdFetchLieferungen";
	public static final String FIND_ANZ_BESTELLUNGEN_BY_PLZ_V1 = PREFIX + "findAnzBestellungenByPlzV1";
	public static final String FIND_ANZ_BESTELLUNGEN_BY_PLZ_V2 = PREFIX + "findAnzBestellungenByPlzV2";
	public static final String FIND_PLZ_ANZAHL_BESTELLUNGEN = PREFIX + "findPlzAnzahlBestellungen";
	
	public static final String PARAM_KUNDEID = "kundeId";
	public static final String PARAM_ID = "id";
	public static final String PARAM_PLZ_MIN = "min";
	public static final String PARAM_PLZ_MAX = "max";

	@Id
	@GeneratedValue(generator = "bestellung_sequence_name")
	@SequenceGenerator(name = "bestellung_sequence_name", sequenceName = "bestellung_b_id_seq", allocationSize = 1)
	@Column(name = "b_id", nullable = false)
	@XmlAttribute(name = "id", required = true)
	private Long id = KEINE_ID;

	@Version
	@XmlTransient
	private int version = ERSTE_VERSION;

	@OneToMany(fetch = EAGER, cascade = { PERSIST, REMOVE })
	@JoinColumn(name = "bestellung_fk", nullable = false)
	@OrderColumn(name = "idx", nullable = false)
	@NotEmpty(message = "{bestellverwaltung.bestellung.bestellpositionen.notEmpty}")
	@Valid
	@XmlElementWrapper(name = "bestellpositionen")
	@XmlElement(name = "bestellposition")
	private List<Bestellposition> bestellpositionen;

	@ManyToOne(optional = false)
	@JoinColumn(name = "kunde_fk", nullable = false, insertable = false, updatable = false)
	@NotNull(message = "{bestellverwaltung.bestellung.kunde.notNull}")
	@XmlTransient
	private AbstractKunde kunde;
	
	@Transient
	@XmlElement(name = "kunde")
	private URI kundeKey;
	
	@ManyToMany
	@JoinTable(name = "bestellung_lieferung",
			   joinColumns = @JoinColumn(name = "bestellung_fk"),
			   inverseJoinColumns = @JoinColumn(name = "lieferung_fk"))
	@OrderColumn(name = "idx", nullable = false)
	@XmlTransient
	private List<Lieferung> lieferungen;
	
	@Transient
	@XmlElementWrapper(name = "lieferungen")
	@XmlElement(name = "lieferung")
	private List<URI> lieferungenKeys;
	
	@OneToOne
	@JoinColumn(name = "geschenkverpackung_fk")
	private Geschenkverpackung geschenkverpackung;
	
	@Column(nullable = false)
	@Temporal(TIMESTAMP)
	private Date erzeugt = null;

	@Column(nullable = false)
	@Temporal(TIMESTAMP)
	private Date aktualisiert = null;
	
	public Bestellung() {
		super();
	}
	
	public Bestellung(AbstractKunde kunde, List<Bestellposition> bestellpositionen) {
		super();
		this.kunde = kunde;
		this.bestellpositionen = bestellpositionen;
	}

	@SuppressWarnings("unused")
	@PrePersist
	private void prePersist() {
		erzeugt = new Date();
		preUpdate();
	}
	
	@SuppressWarnings("unused")
	@PostPersist
	private void logDbId() {
		LOGGER.debug("Neue Bestellung mit ID={}", id);
	}

	@PreUpdate
	private void preUpdate() {
		aktualisiert = new Date();
	}
	
	@SuppressWarnings("unused")
	@PostUpdate
	private void logNeueVersion() {
		LOGGER.trace("Bestellung ID={} geaendert: neue Version={}", id, version);
	}

	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}

	public int getVersion() {
		return version;
	}
	public void setVersion(int version) {
		this.version = version;
	}

	public List<Bestellposition> getBestellpositionen() {
		if (bestellpositionen == null) {
			return null;
		}
		
		return Collections.unmodifiableList(bestellpositionen);
	}
	
	public void setBestellpositionen(List<Bestellposition> bestellpositionen) {
		if (this.bestellpositionen == null) {
			this.bestellpositionen = bestellpositionen;
			return;
		}
		
		// Wiederverwendung der vorhandenen Collection
		this.bestellpositionen.clear();
		if (bestellpositionen != null) {
			this.bestellpositionen.addAll(bestellpositionen);
		}
	}
	
	public Bestellung addBestellposition(Bestellposition bestellposition) {
		if (bestellpositionen == null) {
			bestellpositionen = new ArrayList<Bestellposition>();
		}
		bestellpositionen.add(bestellposition);
		return this;
	}

	public AbstractKunde getKunde() {
		return kunde;
	}
	public void setKunde(AbstractKunde kunde) {
		this.kunde = kunde;
	}

	public URI getKundeKey() {
		return kundeKey;
	}
	
	public void setKundeKey(URI kundeKey) {
		this.kundeKey = kundeKey;
	}

	public List<Lieferung> getLieferungen() {
		if (lieferungen == null) {
			return null;
		}
		
		return Collections.unmodifiableList(lieferungen);
	}
	public void setLieferungen(List<Lieferung> lieferungen) {
		if (this.lieferungen == null) {
			this.lieferungen = lieferungen;
			return;
		}
		
		// Wiederverwendung der vorhandenen Collection
		this.lieferungen.clear();
		if (lieferungen != null) {
			this.lieferungen.addAll(lieferungen);
		}
	}
	
	public void addLieferung(Lieferung lieferung) {
		if (lieferungen == null) {
			lieferungen = new ArrayList<Lieferung>();
		}
		lieferungen.add(lieferung);
	}

	public List<URI> getLieferungenKeys() {
		return lieferungenKeys;
	}
	public void setLieferungenKeys(List<URI> lieferungenKeys) {
		this.lieferungenKeys = lieferungenKeys;
	}

	public Geschenkverpackung getGeschenkverpackung() {
		return geschenkverpackung;
	}
	
	public void setGeschenkverpackung(Geschenkverpackung geschenkverpackung) {
		this.geschenkverpackung = geschenkverpackung;
	}
		
	public Date getErzeugt() {
		return erzeugt == null ? null : (Date) erzeugt.clone();
	}

	public String getErzeugt(String format) {
		final Format formatter = new SimpleDateFormat(format, Locale.getDefault());
		return formatter.format(erzeugt);
	}

	public void setErzeugt(Date erzeugt) {
		this.erzeugt = erzeugt == null ? null : (Date) erzeugt.clone();
	}

	public Date getAktualisiert() {
		return aktualisiert == null ? null : (Date) aktualisiert.clone();
	}

	public void setAktualisiert(Date aktualisiert) {
		this.aktualisiert = aktualisiert == null ? null : (Date) aktualisiert.clone();
	}
	
	@Override
	public String toString() {
		return "Bestellung [id=" + id + ", version=" + version + ", erzeugt=" + erzeugt + ", aktualisiert=" + aktualisiert + ']';
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((bestellpositionen == null) ? 0 : bestellpositionen.hashCode());
		result = prime * result + version;
		result = prime * result + ((erzeugt == null) ? 0 : erzeugt.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final Bestellung other = (Bestellung) obj;
		if (bestellpositionen == null) {
			if (other.bestellpositionen != null) {
				return false;
			}
		}
		else if (!bestellpositionen.equals(other.bestellpositionen)) {
			return false;
		}
		if (version != other.version) {
			return false;
		}
		if (erzeugt == null) {
			if (other.erzeugt != null) {
				return false;
			}
		}
		else if (!erzeugt.equals(other.erzeugt)) {
			return false;
		}
		return true;
	}
}
