package de.swe2.bestellverwaltung.domain;

import static javax.persistence.EnumType.STRING;
import static javax.persistence.TemporalType.TIMESTAMP;
import static javax.xml.bind.annotation.XmlAccessType.FIELD;
import static de.swe2.util.EjbConstants.KEINE_ID;
import static de.swe2.util.EjbConstants.ERSTE_VERSION;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.PostPersist;
import javax.persistence.PostUpdate;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.Version;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlEnum;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


@Entity
@Table(name = "geschenkverpackung")
@XmlRootElement
@XmlAccessorType(FIELD)
public class Geschenkverpackung implements Serializable {
	private static final long serialVersionUID = -6824763459214716853L;
	protected static final Logger LOGGER = LoggerFactory.getLogger(Geschenkverpackung.class);
	
	@XmlEnum
	public enum GeschenkverpackungArt {
		WEIHNACHTEN,
		OSTERN,
		GEBURTSTAG,
		HEIRAT;
	}
	
	@Id
	@GeneratedValue(generator = "geschenkverpackung_sequence_name")
	@SequenceGenerator(name = "geschenkverpackung_sequence_name", sequenceName = "geschenkverpackung_g_id_seq", allocationSize = 1)
	@Column(name = "g_id", nullable = false)
	@XmlTransient
	private Long id = KEINE_ID;

	@Version
	@Column(nullable = false)
	@XmlTransient
	private int version = ERSTE_VERSION;

	@Column(length = 11, nullable = false)
	@Enumerated(STRING)
	@XmlAttribute(required = true)
	private GeschenkverpackungArt art;

	@Column(length = 2000)
	private String text;
	
	@Column(nullable = false)
	@Temporal(TIMESTAMP)
	@XmlTransient
	private Date erzeugt = null;

	@Column(nullable = false)
	@Temporal(TIMESTAMP)
	@XmlTransient
	private Date aktualisiert = null;

	@SuppressWarnings("unused")
	@PrePersist
	private void prePersist() {
		erzeugt = new Date();
		preUpdate();
	}
	
	@SuppressWarnings("unused")
	@PostPersist
	private void logDbId() {
		LOGGER.debug("Neue Geschenkverpackung: {}", this);
	}

	@PreUpdate
	private void preUpdate() {
		aktualisiert = new Date();
	}

	@SuppressWarnings("unused")
	@PostUpdate
	private void logNeueVersion() {
		LOGGER.trace("Geschenkverpackung ID={} geaendert: neue Version={}", id, version);
	}

	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}

	public int getVersion() {
		return version;
	}
	public void setVersion(int version) {
		this.version = version;
	}

	public GeschenkverpackungArt getArt() {
		return art;
	}
	public void setArt(GeschenkverpackungArt art) {
		this.art = art;
	}

	public String getText() {
		return text;
	}
	public void setText(String text) {
		this.text = text;
	}
	
	public Date getErzeugt() {
		return erzeugt == null ? null : (Date) erzeugt.clone();
	}

	public void setErzeugt(Date erzeugt) {
		this.erzeugt = erzeugt == null ? null : (Date) erzeugt.clone();
	}

	public Date getAktualisiert() {
		return aktualisiert == null ? null : (Date) aktualisiert.clone();
	}

	public void setAktualisiert(Date aktualisiert) {
		this.aktualisiert = aktualisiert == null ? null : (Date) aktualisiert.clone();
	}
	
	@Override
	public String toString() {
		return "Geschenkverpackung [id= " + id + ", version=" + version + ", art=" + art
		       + ", text=" + text + ", erzeugt=" + erzeugt + ", aktualisiert=" + aktualisiert + ']';
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((art == null) ? 0 : art.hashCode());
		result = prime * result + ((text == null) ? 0 : text.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final Geschenkverpackung other = (Geschenkverpackung) obj;
		if (art == null) {
			if (other.art != null) {
				return false;
			}
		}
		else if (!art.equals(other.art)) {
			return false;
		}
		if (text == null) {
			if (other.text != null) {
				return false;
			}
		}
		else if (!text.equals(other.text)) {
			return false;
		}
		return true;
	}


}
