package de.swe2.kundenverwaltung.domain;

import static de.swe2.util.EjbConstants.ERSTE_VERSION;
import static de.swe2.util.EjbConstants.KEINE_ID;
import static de.swe2.util.EjbConstants.MIN_ID;
import static javax.persistence.TemporalType.DATE;
import static javax.persistence.TemporalType.TIMESTAMP;
import static javax.xml.bind.annotation.XmlAccessType.FIELD;

import java.net.URI;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.DiscriminatorColumn;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.JoinColumn;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.OrderColumn;
import javax.persistence.PostLoad;
import javax.persistence.PostPersist;
import javax.persistence.PostUpdate;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.SecondaryTable;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.Transient;
import javax.persistence.Version;
import javax.validation.Valid;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Past;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlSeeAlso;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;

import org.hibernate.validator.constraints.Email;
import org.hibernate.validator.constraints.ScriptAssert;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.swe2.bestellverwaltung.domain.Bestellung;
import de.swe2.util.IdGroup;
import de.swe2.util.XmlDateAdapter;


// Alternativen bei @Inheritance
//   strategy = SINGLE_TABLE (=default), TABLE_PER_CLASS, JOINED
@Entity
@Table(name = "kunde")
@SecondaryTable(name = "kunde_details", pkJoinColumns = @PrimaryKeyJoinColumn(name = "kunde_fk"))
@Inheritance
@DiscriminatorColumn(name = "art", length = 1)
@NamedQueries({
	@NamedQuery(name  = AbstractKunde.FIND_KUNDEN,
                query = "SELECT  k"
				        + " FROM AbstractKunde k"),
   	@NamedQuery(name  = AbstractKunde.FIND_KUNDEN_FETCH_BESTELLUNGEN,
                query = "SELECT  DISTINCT k"
				        + " FROM AbstractKunde k LEFT JOIN FETCH k.bestellungen"),
	@NamedQuery(name  = AbstractKunde.FIND_KUNDEN_ORDER_BY_ID,
		        query = "SELECT   k"
				        + " FROM  AbstractKunde k"
		                + " ORDER BY k.id"),
	@NamedQuery(name  = AbstractKunde.FIND_KUNDEN_BY_NACHNAME,
	            query = "SELECT   k"
				        + " FROM  AbstractKunde k"
	            		+ " WHERE UPPER(k.nachname) = UPPER(:" + AbstractKunde.PARAM_KUNDE_NACHNAME + ")"),
	@NamedQuery(name  = AbstractKunde.FIND_KUNDE_BY_EMAIL,
                query = "SELECT   k"
			            + " FROM  AbstractKunde k"
            		    + " WHERE UPPER(k.email) = UPPER(:" + AbstractKunde.PARAM_KUNDE_EMAIL + ")"),
	@NamedQuery(name  = AbstractKunde.FIND_NACHNAMEN_BY_PREFIX,
   	            query = "SELECT   DISTINCT k.nachname"
				        + " FROM  AbstractKunde k "
   	            		+ " WHERE UPPER(k.nachname) LIKE UPPER(:" + AbstractKunde.PARAM_KUNDE_NACHNAME_PREFIX + ")"),
    @NamedQuery(name  = AbstractKunde.FIND_ALL_NACHNAMEN,
   	            query = "SELECT      DISTINCT k.nachname"
   				        + " FROM     AbstractKunde k"
   				        + " ORDER BY k.nachname"),
    @NamedQuery(name  = AbstractKunde.FIND_KUNDEN_BY_NACHNAME_FETCH_BESTELLUNGEN,
	            query = "SELECT      DISTINCT k"
			            + " FROM     AbstractKunde k LEFT JOIN FETCH k.bestellungen"
			            + " WHERE    UPPER(k.nachname) = UPPER(:" + AbstractKunde.PARAM_KUNDE_NACHNAME + ")"
			            + " ORDER BY k.id"),
	@NamedQuery(name  = AbstractKunde.FIND_KUNDE_BY_ID_FETCH_BESTELLUNGEN,
	            query = "SELECT DISTINCT k"
			            + " FROM   AbstractKunde k LEFT JOIN FETCH k.bestellungen"
			            + " WHERE  k.id = :" + AbstractKunde.PARAM_KUNDE_ID),
   	@NamedQuery(name  = AbstractKunde.FIND_KUNDE_BY_ID_FETCH_WARTUNGSVERTRAEGE,
   	            query = "SELECT DISTINCT k"
   			            + " FROM   AbstractKunde k LEFT JOIN FETCH k.wartungsvertraege"
   			            + " WHERE  k.id = :" + AbstractKunde.PARAM_KUNDE_ID),
	@NamedQuery(name  = AbstractKunde.FIND_KUNDEN_BY_PLZ,
	            query = "SELECT   k"
				        + " FROM  AbstractKunde k"
			            + " WHERE k.adresse.plz = :" + AbstractKunde.PARAM_KUNDE_ADRESSE_PLZ),
	@NamedQuery(name  = AbstractKunde.FIND_KUNDE_BY_USERNAME,
	            query = "SELECT   k"
				        + " FROM  AbstractKunde k"
	            		+ " WHERE k.username = :" + AbstractKunde.PARAM_KUNDE_USERNAME),
	@NamedQuery(name = AbstractKunde.FIND_KUNDEN_BY_DATE,
   			    query = "SELECT   k"
   			            + " FROM  AbstractKunde k"
   			    		+ " WHERE k.seit = :" + AbstractKunde.PARAM_KUNDE_SEIT),
   	@NamedQuery(name = AbstractKunde.FIND_PRIVATKUNDEN_FIRMENKUNDEN,
   			    query = "SELECT   k"
   			            + " FROM  AbstractKunde k"
   			    		+ " WHERE TYPE(k) IN (Privatkunde, Firmenkunde)")

})
@ScriptAssert(lang = "javascript",
	          script = "(_this.password == null && _this.passwordWdh == null)"
	                   + "|| (_this.password != null && _this.password.equals(_this.passwordWdh))",
	          groups = AbstractKunde.PasswordGroup.class,
	          message = "{kundenverwaltung.kunde.password.notEqual}")
@XmlRootElement
@XmlSeeAlso({
    Firmenkunde.class,
    Privatkunde.class
})
@XmlAccessorType(FIELD)
public abstract class AbstractKunde implements java.io.Serializable {
	public interface PasswordGroup {
	}
	
	private static final long serialVersionUID = 4190012483483113483L;
	private static final Logger LOGGER = LoggerFactory.getLogger(AbstractKunde.class);
	
	private static final String NAME_PATTERN = "[A-Z\u00C4\u00D6\u00DC][a-z\u00E4\u00F6\u00FC\u00DF]+";
	private static final String PREFIX_ADEL = "(o'|von|von der|von und zu|van)?";
	public static final String NACHNAME_PATTERN = PREFIX_ADEL + NAME_PATTERN + "(-" + NAME_PATTERN + ")?";
	public static final int NACHNAME_LENGTH_MIN = 2;   // kann an der GUI abgefragt werden
	public static final int NACHNAME_LENGTH_MAX = 32;
	public static final int VORNAME_LENGTH_MAX = 32;
	public static final int USERNAME_LENGTH_MAX = 32;
	public static final int PASSWORD_LENGTH_MAX = 256;
	public static final int DETAILS_LENGTH_MAX = 128 * 1024;
	public static final int EMAIL_LENGTH_MAX = 32;
	public static final int BEMERKUNGEN_LENGTH_MAX = 2000;
	
	public static final String PRIVATKUNDE = "P";
	public static final String FIRMENKUNDE = "F";
	
	private static final String PREFIX = "AbstractKunde.";
	public static final String FIND_KUNDEN = PREFIX + "findKunden";
	public static final String FIND_KUNDEN_FETCH_BESTELLUNGEN = PREFIX + "findKundenFetchBestellungen";
	public static final String FIND_KUNDEN_ORDER_BY_ID = PREFIX + "findKundenOrderById";
	public static final String FIND_KUNDEN_BY_NACHNAME = PREFIX + "findKundenByNachname";
	public static final String FIND_KUNDE_BY_EMAIL = PREFIX + "findKundeByEmail";
	public static final String FIND_NACHNAMEN_BY_PREFIX = PREFIX + "findNachnamenByPrefix";
	public static final String FIND_ALL_NACHNAMEN = PREFIX + "findAllNachnamen";
	public static final String FIND_KUNDEN_BY_NACHNAME_FETCH_BESTELLUNGEN = PREFIX + "findKundenByNachnameFetchBestellungen";
	public static final String FIND_KUNDE_BY_ID_FETCH_BESTELLUNGEN = PREFIX + "findKundeByIdFetchBestellungen";
	public static final String FIND_KUNDE_BY_ID_FETCH_WARTUNGSVERTRAEGE = PREFIX + "findKundenByNachnameFetchWartungsvertraege";
	public static final String FIND_KUNDEN_BY_PLZ = PREFIX + "findKundenByPlz";
	public static final String FIND_KUNDE_BY_USERNAME = PREFIX + "findKundeByUsername";
	public static final String FIND_KUNDEN_BY_DATE = PREFIX + "findKundenByDate";
	public static final String FIND_PRIVATKUNDEN_FIRMENKUNDEN = PREFIX + "findPrivatkundenFirmenkunden";

	public static final String PARAM_KUNDE_ID = "id";
	public static final String PARAM_KUNDE_NACHNAME = "nachname";
	public static final String PARAM_KUNDE_NACHNAME_PREFIX = "nachnamePrefix";
	public static final String PARAM_KUNDE_ADRESSE_PLZ = "plz";
	public static final String PARAM_KUNDE_USERNAME = "username";
	public static final String PARAM_KUNDE_EMAIL = "email";
	public static final String PARAM_KUNDE_SEIT = "seit";

	
	@Id
	@GeneratedValue(generator = "kunde_sequence_name")
	@SequenceGenerator(name = "kunde_sequence_name", sequenceName = "kunde_k_id_seq", allocationSize = 1)
	@Column(name = "k_id", nullable = false)
	@Min(value = MIN_ID, message = "{kundenverwaltung.kunde.id.min}", groups = IdGroup.class)
	@XmlAttribute(name = "id", required = true)
	private Long id = KEINE_ID;

	@Version
	@XmlTransient
	private int version = ERSTE_VERSION;

	@Column(length = NACHNAME_LENGTH_MAX, nullable = false)
	@NotNull(message = "{kundenverwaltung.kunde.nachname.notNull}")
	@Size(min = NACHNAME_LENGTH_MIN, max = NACHNAME_LENGTH_MAX, message = "{kundenverwaltung.kunde.nachname.length}")
	@Pattern(regexp = NACHNAME_PATTERN, message = "{kundenverwaltung.kunde.nachname.pattern}")
	@XmlElement(required = true)
	private String nachname = "";

	@Column(length = VORNAME_LENGTH_MAX)
	@Size(max = VORNAME_LENGTH_MAX, message = "{kundenverwaltung.kunde.vorname.length}")
	private String vorname = "";
	
	@Column(nullable = false)
	private short kategorie;
	
	@Column(nullable = false)
	private float rabatt;
	
	@Column(nullable = false)
	private double umsatz;
	
	@Temporal(DATE)
	@Past(message = "{kundenverwaltung.kunde.seit.past}")
	@XmlJavaTypeAdapter(XmlDateAdapter.class) 
	private Date seit;
	
	@Column(length = USERNAME_LENGTH_MAX, unique = true)
	@Size(max = USERNAME_LENGTH_MAX, message = "{kundenverwaltung.kunde.username.length}")
	private String username = "";
	
	@Column(length = EMAIL_LENGTH_MAX, unique = true)
	@Email(message = "{kundenverwaltung.kunde.email}")
	private String email = "";
	
	private boolean newsletter = false;

	@Column(length = PASSWORD_LENGTH_MAX)
	@Size(max = PASSWORD_LENGTH_MAX, message = "{kundenverwaltung.kunde.password.length}")
	private String password;
	
	@Transient
	@XmlTransient
	private String passwordWdh;

//  siehe @ScriptAssert
//	@AssertTrue(groups = PasswordGroup.class, message = "{kundenverwaltung.kunde.password.notEqual}")
//	public boolean isPasswortEqual() {
//		if (password == null) {
//			return passwordWdh == null;
//		}
//		return password.equals(passwordWdh);
//	}

	@Embedded
	@Valid
	@NotNull(message = "{kundenverwaltung.kunde.adresse.notNull}")
	private Adresse adresse;

	@OneToOne(mappedBy = "kunde")
	@XmlTransient
	private Betreuer betreuer;
	
	// Default: fetch = LAZY, keine Kaskadierungen
	// Alternativen: cascade = PERSIST, cascade = { PERSIST, REMOVE } usw.
	@OneToMany
	@JoinColumn(name = "kunde_fk", nullable = false)
	@OrderColumn(name = "idx", nullable = false)
	@XmlTransient
	private List<Bestellung> bestellungen;
	
	@Transient
	@XmlElementWrapper(name = "bestellungen")
	@XmlElement(name = "bestellung")
	private List<URI> bestellungenKeys;
	
	@Column(table = "kunde_details", length = DETAILS_LENGTH_MAX)
	private String details;

	
	@OneToMany
	@JoinColumn(name = "kunde_fk", nullable = false)
	@OrderColumn(name = "idx", nullable = false)
	@XmlTransient
	private List<Wartungsvertrag> wartungsvertraege;
	
	@Column(length = 128)
	private String dateiname;
	
	@Column(name = "datei_content_type", length = 64)
	private String dateiContentType;
	
	@Column
	@Size(max = BEMERKUNGEN_LENGTH_MAX)
	private String bemerkungen;
	
	@Column(nullable = false)
	@Temporal(TIMESTAMP)
	private Date erzeugt = null;

	@Column(nullable = false)
	@Temporal(TIMESTAMP)
	private Date aktualisiert = null;
	
	public AbstractKunde() {
		super();
	}

	public AbstractKunde(String nachname, String vorname, String email, Date seit) {
		super();
		this.nachname = nachname;
		this.vorname = vorname;
		this.email = email;
		this.seit = seit == null ? null : (Date) seit.clone();
	}
	
	@PrePersist
	protected void prePersist() {
		erzeugt = new Date();
		preUpdate();
	}
	
	@PostPersist
	protected void postPersist() {
		setUsername(getId().toString());

		LOGGER.debug("Neuer Kunde mit ID={}", id);
	}

	@PreUpdate
	protected void preUpdate() {
		aktualisiert = new Date();
	}
	
	@PostUpdate
	protected void postUpdate() {
		LOGGER.trace("Kunde mit ID={} hat die neue Version {}", id, version);
	}
	
	@PostLoad
	protected void postLoad() {
		passwordWdh = password;
	}
	
	public void setValues(AbstractKunde k) {
		nachname = k.getNachname();
		vorname = k.getVorname();
		kategorie = k.getKategorie();
		umsatz = k.getUmsatz();
		seit = k.getSeit();
		username = k.getUsername();
		email = k.getEmail();
		dateiname = k.getDateiname();
		dateiContentType = k.getDateiContentType();
		password = k.getPassword();
		passwordWdh = k.getPassword();
	}

	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}

	public int getVersion() {
		return version;
	}
	public void setVersion(int version) {
		this.version = version;
	}

	public String getNachname() {
		return nachname;
	}
	public void setNachname(String nachname) {
		this.nachname = nachname;
	}

	public String getVorname() {
		return vorname;
	}
	public void setVorname(String vorname) {
		this.vorname = vorname;
	}
	
	public short getKategorie() {
		return kategorie;
	}

	public void setKategorie(short kategorie) {
		this.kategorie = kategorie;
	}
	
	public float getRabatt() {
		return rabatt;
	}
	public void setRabatt(float rabatt) {
		this.rabatt = rabatt;
	}

	public double getUmsatz() {
		return umsatz;
	}

	public void setUmsatz(double umsatz) {
		this.umsatz = umsatz;
	}

	public Date getSeit() {
		return seit == null ? null : (Date) seit.clone();
	}

	public void setSeit(Date seit) {
		this.seit = seit == null ? null : (Date) seit.clone();
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}
	
	public void setNewsletter(boolean newsletter) {
		this.newsletter = newsletter;
	}

	public boolean isNewsletter() {
		return newsletter;
	}

	
	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getPasswordWdh() {
		return passwordWdh;
	}

	public void setPasswordWdh(String passwordWdh) {
		this.passwordWdh = passwordWdh;
	}
	
	public Adresse getAdresse() {
		return adresse;
	}
	public void setAdresse(Adresse adresse) {
		this.adresse = adresse;
	}

	public Betreuer getBetreuer() {
		return betreuer;
	}

	public void setBetreuer(Betreuer betreuer) {
		this.betreuer = betreuer;
	}

	public List<Bestellung> getBestellungen() {
		if (bestellungen == null) {
			return null;
		}
		return Collections.unmodifiableList(bestellungen);
	}
	public void setBestellungen(List<Bestellung> bestellungen) {
		if (this.bestellungen == null) {
			this.bestellungen = bestellungen;
			return;
		}
		
		// Wiederverwendung der vorhandenen Collection
		this.bestellungen.clear();
		if (bestellungen != null) {
			this.bestellungen.addAll(bestellungen);
		}
	}
	
	public AbstractKunde addBestellung(Bestellung bestellung) {
		if (bestellungen == null) {
			bestellungen = new ArrayList<Bestellung>();
		}
		bestellungen.add(bestellung);
		return this;
	}

	public List<URI> getBestellungenKeys() {
		return bestellungenKeys;
	}
	public void setBestellungenKeys(List<URI> bestellungenKeys) {
		this.bestellungenKeys = bestellungenKeys;
	}

	public String getDetails() {
		return details;
	}

	public void setDetails(String details) {
		this.details = details;
	}

	public List<Wartungsvertrag> getWartungsvertraege() {
		if (wartungsvertraege == null) {
			return null;
		}
		
		return Collections.unmodifiableList(wartungsvertraege);
	}

	public void setWartungsvertraege(List<Wartungsvertrag> wartungsvertraege) {
		if (this.wartungsvertraege == null) {
			this.wartungsvertraege = wartungsvertraege;
			return;
		}
		
		// Wiederverwendung der vorhandenen Collection
		this.wartungsvertraege.clear();
		if (wartungsvertraege != null) {
			this.wartungsvertraege.addAll(wartungsvertraege);
		}
	}
	
	public AbstractKunde addWartungsvertrag(Wartungsvertrag wartungsvertrag) {
		if (wartungsvertraege == null) {
			wartungsvertraege = new ArrayList<Wartungsvertrag>();
		}
		wartungsvertraege.add(wartungsvertrag);
		return this;
	}

	public abstract String getArt();

	public String getDateiname() {
		return dateiname;
	}

	public void setDateiname(String dateiname) {
		this.dateiname = dateiname;
	}

	public String getDateiContentType() {
		return dateiContentType;
	}

	public void setDateiContentType(String dateiContentType) {
		this.dateiContentType = dateiContentType;
	}

	public String getBemerkungen() {
		return bemerkungen;
	}

	public void setBemerkungen(String bemerkungen) {
		this.bemerkungen = bemerkungen;
	}

	public Date getErzeugt() {
		return erzeugt == null ? null : (Date) erzeugt.clone();
	}

	public void setErzeugt(Date erzeugt) {
		this.erzeugt = erzeugt == null ? null : (Date) erzeugt.clone();
	}

	public Date getAktualisiert() {
		return aktualisiert == null ? null : (Date) aktualisiert.clone();
	}

	public void setAktualisiert(Date aktualisiert) {
		this.aktualisiert = aktualisiert == null ? null : (Date) aktualisiert.clone();
	}

	@Override
	public String toString() {
		return "Kunde [id=" + id + ", version=" + version
			   + ", nachname=" + nachname + ", vorname=" + vorname + ", seit=" + seit
			   + ", email=" + email + ", kategorie=" + kategorie + ", rabatt=" + rabatt + ", art=" + getArt()
			   + ", username=" + username + ", password=" + password + ", dateiname=" + dateiname
			   + ", passwordWdh=" + passwordWdh
			   + ", erzeugt=" + erzeugt
			   + ", aktualisiert=" + aktualisiert + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((email == null) ? 0 : email.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final AbstractKunde other = (AbstractKunde) obj;
		if (email == null) {
			if (other.email != null) {
				return false;
			}
		}
		else if (!email.equals(other.email)) {
			return false;
		}
		return true;
	}
}
