package de.swe2.kundenverwaltung.domain;

import static de.swe2.util.EjbConstants.ERSTE_VERSION;
import static de.swe2.util.EjbConstants.KEINE_ID;
import static javax.persistence.TemporalType.TIMESTAMP;
import static javax.xml.bind.annotation.XmlAccessType.FIELD;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToOne;
import javax.persistence.PostPersist;
import javax.persistence.PostUpdate;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.Version;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


@Entity
@Table(name = "betreuer")
@XmlRootElement
@XmlAccessorType(FIELD)
public class Betreuer implements Serializable {
	private static final long serialVersionUID = -4090693388150752068L;
	protected static final Logger LOGGER = LoggerFactory.getLogger(Betreuer.class);

	@Id
	@GeneratedValue(generator = "betreuer_sequence_name")
	@SequenceGenerator(name = "betreuer_sequence_name", sequenceName = "betreuer_b_id_seq", allocationSize = 1)
	@Column(name = "b_id", nullable = false)
	@XmlAttribute(name = "id", required = true)
	private Long id = KEINE_ID;

	@Version
	@XmlTransient
	private int version = ERSTE_VERSION;

	@Column(length = 32, nullable = false)
	private String name = "";

	@OneToOne
	@JoinColumn(name = "kunde_fk")
	@XmlTransient
	private AbstractKunde kunde = null;
	
	@Column(nullable = false)
	@Temporal(TIMESTAMP)
	private Date erzeugt = null;

	@Column(nullable = false)
	@Temporal(TIMESTAMP)
	private Date aktualisiert = null;
	
	@SuppressWarnings("unused")
	@PrePersist
	private void prePersist() {
		erzeugt = new Date();
		preUpdate();
	}
	
	@SuppressWarnings("unused")
	@PostPersist
	private void logDbId() {
		LOGGER.debug("Neuer Betreuer: {}", this);
	}

	@PreUpdate
	private void preUpdate() {
		aktualisiert = new Date();
	}
	
	@SuppressWarnings("unused")
	@PostUpdate
	private void logNeueVersion() {
		LOGGER.trace("Betreuer ID={} geaendert: neue Version={}", id, version);
	}
	
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}

	public int getVersion() {
		return version;
	}
	public void setVersion(int version) {
		this.version = version;
	}

	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}

	public AbstractKunde getKunde() {
		return kunde;
	}
	public void setKunde(AbstractKunde kunde) {
		this.kunde = kunde;
	}
	
	public Date getErzeugt() {
		return erzeugt == null ? null : (Date) erzeugt.clone();
	}

	public void setErzeugt(Date erzeugt) {
		this.erzeugt = erzeugt == null ? null : (Date) erzeugt.clone();
	}

	public Date getAktualisiert() {
		return aktualisiert == null ? null : (Date) aktualisiert.clone();
	}

	public void setAktualisiert(Date aktualisiert) {
		this.aktualisiert = aktualisiert == null ? null : (Date) aktualisiert.clone();
	}
	
	@Override
	public String toString() {
		return "Betreuer [id=" + id + ", name=" + name
		       + ", erzeugt=" + erzeugt + ", aktualisiert=" + aktualisiert + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((kunde == null) ? 0 : kunde.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final Betreuer other = (Betreuer) obj;
		if (kunde == null) {
			if (other.kunde != null) {
				return false;
			}
		}
		else if (!kunde.equals(other.kunde)) {
			return false;
		}
		if (name == null) {
			if (other.name != null) {
				return false;
			}
		}
		else if (!name.equals(other.name)) {
			return false;
		}
		return true;
	}
}
