package de.swe2.kundenverwaltung.domain;


import static de.swe2.kundenverwaltung.domain.AbstractKunde.PRIVATKUNDE;
import static javax.xml.bind.annotation.XmlAccessType.FIELD;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;
import javax.persistence.Inheritance;
import javax.persistence.PostLoad;
import javax.persistence.PrePersist;
import javax.persistence.Transient;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlEnum;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


@Entity
@Inheritance
@DiscriminatorValue(PRIVATKUNDE)
@XmlRootElement
@XmlAccessorType(FIELD)
public final class Privatkunde extends AbstractKunde {
	private static final long serialVersionUID = -1783340753647408724L;
	private static final Logger LOGGER = LoggerFactory.getLogger(Privatkunde.class);

	@XmlEnum
	public enum Familienstand {
		LEDIG,
		VERHEIRATET,
		GESCHIEDEN,
		VERWITWET;
	}
	
	@XmlEnum
	public enum Geschlecht {
		MAENNLICH,
		WEIBLICH;
	}
	
	@XmlEnum
	public enum Hobby {
		SPORT,
		LESEN,
		REISEN;
	}
	
	private Familienstand familienstand = Familienstand.LEDIG;
	
	private Geschlecht geschlecht = Geschlecht.WEIBLICH;
	
	@Transient
	@XmlElementWrapper
	@XmlElement(name = "hobby")
	private List<Hobby> hobbies;
	
	@Column(name = "hobbies")
	@XmlTransient
	private String hobbiesDB;
	
	public Privatkunde() {
		super();
	}
	
	public Privatkunde(String nachname, String vorname, String email, Date seit) {
		super(nachname, vorname, email, seit);
	}
	
	@PrePersist
	@Override
	protected void prePersist() {
		LOGGER.debug("BEGINN Privatkunde.prePersist: " +  this);
		super.prePersist();
		hobbiesToDB();
		LOGGER.debug("ENDE Privatkunde.prePersist: " +  this);
	}
	
//	TODO @PreUpdate wird bei Privatkunde nicht aufgerufen
//	@PreUpdate
//	@Override
//	protected void preUpdate() {
//		LOGGER.debug("BEGINN Privatkunde.preUpdate: " +  this);
//		super.preUpdate();
//		hobbiesToDB();
//		LOGGER.debug("ENDE Privatkunde.preUpdate: " +  this);
//	}
	
	public void hobbiesToDB() {
		LOGGER.debug("BEGINN hobbiesToDB: {}", this);
		if (hobbies == null || hobbies.isEmpty()) {
			hobbiesDB = "";
			return;
		}

		// Von der Oberflaeche kommen Strings
		final StringBuilder sb = new StringBuilder();
		for (Object hobby : hobbies) {
			if (hobby == null) {
				continue;
			}
			sb.append(hobby);
			sb.append(",");
		}
		final int length = sb.length();
		sb.delete(length - 1, length);   // letztes Komma entfernen
		hobbiesDB = sb.toString();
		LOGGER.debug("ENDE hobbiesToDB: {}", this);
	}
	
	@PostLoad
	@Override
	protected void postLoad() {
		LOGGER.debug("BEGINN Privatkunde.postload: " +  this);
		super.postLoad();
		
		if (hobbiesDB == null || hobbiesDB.isEmpty()) {
			hobbies = new ArrayList<Hobby>();
			return;
		}
		
		String[] tokens = hobbiesDB.split(",");
		hobbies = new ArrayList<Hobby>();
		for (String token : tokens) {
			try {
				hobbies.add(Hobby.valueOf(token));
			}
			catch (IllegalArgumentException e) {
				LOGGER.warn("{} ist kein gueltiger Wert fuer ein Hobby des Kunden {}", token, this);
			}
		}
		
		LOGGER.debug("ENDE Privatkunde.postload: " +  this);
	}
	
	public Familienstand getFamilienstand() {
		return familienstand;
	}
	
	public void setFamilienstand(Familienstand familienstand) {
		this.familienstand = familienstand;
	}

	public Geschlecht getGeschlecht() {
		return geschlecht;
	}
	public void setGeschlecht(Geschlecht geschlecht) {
		this.geschlecht = geschlecht;
	}
	public List<Hobby> getHobbies() {
		return hobbies;
	}
	
	public void setHobbies(List<Hobby> hobbies) {
		this.hobbies = hobbies;
	}

	public String getHobbiesDB() {
		return hobbiesDB;
	}

	public void setHobbiesDB(String hobbiesDB) {
		this.hobbiesDB = hobbiesDB;
	}

	@Override
	public String getArt() {
		return PRIVATKUNDE;
	}
	
	@Override
	public String toString() {
		return "Privatkunde [" + super.toString() + ", familienstand=" + familienstand
		       + ", geschlecht=" + geschlecht + ", hobbies=" + hobbies + ", hobbiesDB=" + hobbiesDB + "]";
	}
}
