package de.swe2.kundenverwaltung.domain;

import static javax.persistence.TemporalType.DATE;
import static javax.persistence.TemporalType.TIMESTAMP;
import static javax.xml.bind.annotation.XmlAccessType.FIELD;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Embeddable;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.IdClass;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.PostUpdate;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.Version;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


@Entity
@Table(name = "wartungsvertrag")
@NamedQueries({
	@NamedQuery(name  = Wartungsvertrag.FIND_WARTUNGSVERTRAEGE_BY_KUNDE_ID,
                query = "SELECT w"
				        + " FROM   Wartungsvertrag w"
			            + " WHERE  w.kunde.id = :" + Wartungsvertrag.PARAM_KUNDE_ID)
})
@IdClass(Wartungsvertrag.WartungsvertragId.class)
@XmlRootElement
@XmlAccessorType(FIELD)
public class Wartungsvertrag implements Serializable {
	private static final long serialVersionUID = -5955263122430830600L;
	protected static final Logger LOGGER = LoggerFactory.getLogger(Wartungsvertrag.class);
	
	public static final String FIND_WARTUNGSVERTRAEGE_BY_KUNDE_ID = "Wartungsvertrag.findWartungsvertraegeByKundeId";
	public static final String PARAM_KUNDE_ID = "kundeId";

	@Id
	@Column(nullable = false)
	@XmlAttribute(name = "id", required = true)
	private long nr;

	@Id
	@Column(nullable = false)
	@Temporal(DATE)
	@XmlAttribute(name = "datum", required = true)
	private Date datum;

	@Version
	@XmlTransient
	private int version = 0;
	
	@Column
	private String inhalt = null;

	@ManyToOne(optional = false)
	@JoinColumn(name = "kunde_fk", nullable = false, insertable = false, updatable = false)
	@XmlTransient
	private AbstractKunde kunde = null;
	
	@Column(nullable = false)
	@Temporal(TIMESTAMP)
	private Date erzeugt = null;

	@Column(nullable = false)
	@Temporal(TIMESTAMP)
	private Date aktualisiert = null;
	
	public Wartungsvertrag() {
		super();
	}
	
	public Wartungsvertrag(long nr, Date datum) {
		super();
		this.nr = nr;
		this.datum = datum == null ? null : (Date) datum.clone();
	}

	@SuppressWarnings("unused")
	@PrePersist
	private void prePersist() {
		erzeugt = new Date();
		preUpdate();
	}
	
	@PreUpdate
	private void preUpdate() {
		aktualisiert = new Date();
	}
	
	@SuppressWarnings("unused")
	@PostUpdate
	private void logNeueVersion() {
		LOGGER.trace("Wartungsvertrag nr={}, datum={} geaendert: neue Version={}", new Object[] {nr, datum, version});
	}

	public long getNr() {
		return nr;
	}

	public void setNr(long nr) {
		this.nr = nr;
	}

	public Date getDatum() {
		return datum == null ? null : (Date) datum.clone();
	}

	public void setDatum(Date datum) {
		this.datum = datum == null ? null : (Date) datum.clone();
	}

	public int getVersion() {
		return version;
	}

	public void setVersion(int version) {
		this.version = version;
	}

	public String getInhalt() {
		return inhalt;
	}

	public void setInhalt(String inhalt) {
		this.inhalt = inhalt;
	}

	public AbstractKunde getKunde() {
		return kunde;
	}

	public void setKunde(AbstractKunde kunde) {
		this.kunde = kunde;
	}
	
	public Date getErzeugt() {
		return erzeugt == null ? null : (Date) erzeugt.clone();
	}

	public void setErzeugt(Date erzeugt) {
		this.erzeugt = erzeugt == null ? null : (Date) erzeugt.clone();
	}

	public Date getAktualisiert() {
		return aktualisiert == null ? null : (Date) aktualisiert.clone();
	}

	public void setAktualisiert(Date aktualisiert) {
		this.aktualisiert = aktualisiert == null ? null : (Date) aktualisiert.clone();
	}

	/**
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((datum == null) ? 0 : datum.hashCode());
		result = prime * result + (int) (nr ^ (nr >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		Wartungsvertrag other = (Wartungsvertrag) obj;
		if (datum == null) {
			if (other.datum != null) {
				return false;
			}
		}
		else if (!datum.equals(other.datum)) {
			return false;
		}
		if (nr != other.nr) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {
		return "Wartungsvertrag [nr=" + nr + ", datum=" + datum + ", version=" + version + ", inhalt=" + inhalt + ']';
	}


	/**
	 */
	@Embeddable
	public static class WartungsvertragId implements Serializable {
		private static final long serialVersionUID = 9155270685609346099L;

		private long nr = -1;
		private Date datum = new Date();

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((datum == null) ? 0 : datum.hashCode());
			result = prime * result + (int) (nr ^ (nr >>> 32));
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj) {
				return true;
			}
			if (obj == null) {
				return false;
			}
			if (getClass() != obj.getClass()) {
				return false;
			}
			final WartungsvertragId other = (WartungsvertragId) obj;
			if (nr != other.nr) {
				return false;
			}
			if (datum == null) {
				if (other.datum != null) {
					return false;
				}
			}
			else if (!datum.equals(other.datum)) {
				return false;
			}
			return true;
		}

		public long getNr() {
			return nr;
		}

		public void setNr(long nr) {
			this.nr = nr;
		}

		public Date getDatum() {
			return datum == null ? null : (Date) datum.clone();
		}

		public void setDatum(Date datum) {
			this.datum = datum == null ? null : (Date) datum.clone();
		}
	}
}
