package de.swe2.kundenverwaltung.service;

import java.util.Collection;

import javax.ejb.ApplicationException;
import javax.validation.ConstraintViolation;

import de.swe2.kundenverwaltung.domain.AbstractKunde;

@ApplicationException(rollback = true)
public class KundeValidationException extends AbstractKundenverwaltungException {
	private static final long serialVersionUID = 4255133082483647701L;

	private final Collection<ConstraintViolation<AbstractKunde>> violations;
	private Long id;
	private String nachname;
	private String vorname;

	public KundeValidationException(AbstractKunde kunde, Collection<ConstraintViolation<AbstractKunde>> violations) {
		super("Ungueltige Attributwerte");
		if (kunde != null) {
			this.id = kunde.getId();
			this.nachname = kunde.getNachname();
			this.vorname = kunde.getVorname();
		}
		this.violations = violations;
	}
	
	
	public KundeValidationException(Long id, Collection<ConstraintViolation<AbstractKunde>> violations) {
		super("Ungueltige ID");
		this.id = id;
		this.violations = violations;
	}
	
	public KundeValidationException(String nachname, Collection<ConstraintViolation<AbstractKunde>> violations) {
		super("Ungueltiger Nachname");
		this.nachname = nachname;
		this.violations = violations;
	}
	
//	protected void setMessages(Collection<ConstraintViolation<Kunde>> violations) {
//		messages = new ArrayList<String>(violations.size());
//		for (ConstraintViolation<Kunde> v: violations) {
//			messages.add(v.getMessage());
//		}
//	}

	public Long getId() {
		return id;
	}
	public String getNachname() {
		return nachname;
	}
	public String getVorname() {
		return vorname;
	}
	
	public Collection<ConstraintViolation<AbstractKunde>> getViolations() {
		return violations;
	}
	
	@Override
	public String toString() {
		return "{nachname=" + nachname + ", vorname=" + vorname + ", violations=" + violations + "}";
	}
}
