package de.swe2.util.jboss;

import java.security.Principal;

import javax.ejb.EJBException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.jboss.security.SimplePrincipal;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public final class RolesUtil {
	private static final Logger LOGGER = LoggerFactory.getLogger(RolesUtil.class);
	
	private static final String LOGIN_DOMAIN = "swe2";  // siehe security-policy-jboss-beans.xml
	
	private static final ObjectName JAAS_SECURITY_MANAGER = getJaasSecurityManager();
	private static final MBeanServer SERVER = MBeanServerFactory.findMBeanServer(null).get(0);
	private static final String[] SIGNATURE = {String.class.getName(), Principal.class.getName()};
	
	private static ObjectName getJaasSecurityManager() {
		ObjectName jaasSecurityManager = null;
		try {
			jaasSecurityManager = new ObjectName("jboss.security:service=JaasSecurityManager");
		}
		catch (MalformedObjectNameException e) {
			throw new EJBException(e);
		}
		
		return jaasSecurityManager;
	}
	
	/**
	 * Keine Objekte von dieser Klasse erzeugen
	 */
	private RolesUtil() { }
	
	/**
	 */
	// TODO Portierung: Wie werden Rollen im laufenden Betrieb geaendert?
	public static void refreshRoles(String username) {
		LOGGER.debug("BEGIN refreshRoles: username={}" +  username);
		
		final Principal user = new SimplePrincipal(username);
		final Object[] params = {LOGIN_DOMAIN, user};
		
		try {
			SERVER.invoke(JAAS_SECURITY_MANAGER, "flushAuthenticationCache", params, SIGNATURE);
		}
		catch (InstanceNotFoundException e) {
			throw new EJBException(e);
		}
		catch (ReflectionException e) {
			throw new EJBException(e);
		}
		catch (MBeanException e) {
			throw new EJBException(e);
		}
		
		LOGGER.debug("END refreshRoles");
	}
}
