package de.swe2.admin.ui;

import static de.swe2.util.WebConstants.JSF_INDEX;
import static javax.faces.application.FacesMessage.SEVERITY_ERROR;

import java.io.Serializable;

import javax.enterprise.context.SessionScoped;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;

import de.swe2.util.Log;
import de.swe2.util.MessagesMgdBean;
import de.swe2.util.MessagesMgdBean.MessagesType;


/**
 * Ein Managed Bean zur Abwicklung von Authentifizierung (Login und Logout) und erweiterbar f&uuml;r
 * Authorisierung (rollenbasierte Berechtigungen).
 */
@Named("auth")
@SessionScoped
@Log
public class AuthMgdBean implements Serializable {
	private static final long serialVersionUID = -8604525347843804815L;
	
	private static final String MSG_KEY_LOGIN_ERROR = "login.error";
	private static final String CLIENT_ID_USERNAME = "loginForm:username";
	
	@Inject
	private MessagesMgdBean messages;
	
	private String username;
	private String password;

	@Override
	public String toString() {
		return "AuthMgdBean [username=" + username + ", password=" + password + "]";
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	/**
	 * Einloggen eines registrierten Kunden mit Benutzername und Password.
	 */
	public String login() {
		final FacesContext ctx = FacesContext.getCurrentInstance();
		final HttpServletRequest request = (HttpServletRequest) ctx.getExternalContext().getRequest();
		try {
			request.login(username, password);
		}
		catch (ServletException e) {
			// zuruecksetzen
			reset();
			messages.createFacesMessage(MessagesType.SWE2,
					                    MSG_KEY_LOGIN_ERROR,
					                    CLIENT_ID_USERNAME,
					                    SEVERITY_ERROR);
			return null;   // Gleiche Seite nochmals aufrufen: mit den fehlerhaften Werten
		}
		
		return null;
	}
	
	/**
	 */
	private void reset() {
		username = null;
		password = null;
	}

	
	/**
	 * Ausloggen und L&ouml;schen der gesamten Session-Daten.
	 */
	public String logout() {
		final ExternalContext ctx = FacesContext.getCurrentInstance().getExternalContext();
		final HttpServletRequest request = (HttpServletRequest) ctx.getRequest();
		try {
			request.logout();  // Der Loginname wird zurueckgesetzt
		}
		catch (ServletException e) {
			return null;
		}
		
		ctx.invalidateSession();
		reset();
		
		return JSF_INDEX;
	}
	
	public void login_j_security_check() {
		// Keine Verwendung von Request.login(),
		// da j_security_check bereits das Einloggen uebernommen hat
		
		final String remoteUserName = FacesContext.getCurrentInstance().getExternalContext().getRemoteUser();
		if (remoteUserName == null) {
			return;
		}
		
		username = remoteUserName;
	}

	/**
	 * &Uuml;berpr&uuml;fen, ob Login-Informationen vorhanden sind.
	 * @return true, falls man eingeloggt ist.
	 */
	public boolean isLoggedIn() {
		return username != null;
	}
}
