package de.swe2.artikelverwaltung.ui;

import static javax.faces.application.FacesMessage.SEVERITY_ERROR;

import java.io.Serializable;
import java.util.List;

import javax.ejb.EJB;
import javax.enterprise.context.RequestScoped;
import javax.faces.context.FacesContext;
import javax.faces.context.Flash;
import javax.inject.Inject;
import javax.inject.Named;

import de.swe2.artikelverwaltung.domain.Artikel;
import de.swe2.artikelverwaltung.service.Artikelverwaltung;
import de.swe2.util.Log;
import de.swe2.util.MessagesMgdBean;
import de.swe2.util.MessagesMgdBean.MessagesType;
import de.swe2.util.NotFoundException;
import de.swe2.util.RequiredTx;


/**
 * Dialogsteuerung fuer die Artikelverwaltung
 */
@Named("av")
@RequestScoped
@RequiredTx
@Log
public class ArtikelverwaltungMgdBean implements Serializable {
	private static final long serialVersionUID = 1564024850446471639L;
	
	private static final String JSF_LIST_ARTIKEL = "/artikelverwaltung/listArtikel";
	
	private static final String FLASH_ARTIKEL = "artikel";
	
	private static final String CLIENT_ID_BEZEICHNUNG = "findForm:bezeichnung";
	private static final String MSG_KEY_ARTIKEL_NOT_FOUND = "listArtikel.notFound";
	
	@EJB
	private Artikelverwaltung av;
	
	@Inject
	private MessagesMgdBean messages;   // get/set notwendig fuer @ManagedProperty
	
	private String bezeichnung;
		
	@Override
	public String toString() {
		return "ArtikelverwaltungMgdBean [bezeichnung=" + bezeichnung + "]";
	}

	public String getBezeichnung() {
		return bezeichnung;
	}

	public void setBezeichnung(String bezeichnung) {
		this.bezeichnung = bezeichnung;
	}

	/**
	 */
	public String findArtikelByBezeichnung() {
		List<Artikel> artikel;
		try {
			artikel = av.findArtikelByBezeichnung(bezeichnung);
		}
		catch (NotFoundException e) {
			messages.createFacesMessage(MessagesType.ARTIKELVERWALTUNG,
					                    MSG_KEY_ARTIKEL_NOT_FOUND,
					                    CLIENT_ID_BEZEICHNUNG,
					                    SEVERITY_ERROR,
					                    bezeichnung);
			return null;
		}
		
		final Flash flash = FacesContext.getCurrentInstance().getExternalContext().getFlash();
		flash.put(FLASH_ARTIKEL, artikel);
		
		return JSF_LIST_ARTIKEL;
	}
}
