package de.swe2.bestellverwaltung.ui;

import static de.swe2.util.WebConstants.JSF_DEFAULT_ERROR;
import static javax.faces.application.FacesMessage.SEVERITY_ERROR;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.ejb.EJB;
import javax.enterprise.context.SessionScoped;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.context.Flash;
import javax.inject.Inject;
import javax.inject.Named;

import de.swe2.admin.ui.AuthMgdBean;
import de.swe2.artikelverwaltung.domain.Artikel;
import de.swe2.bestellverwaltung.domain.Bestellposition;
import de.swe2.bestellverwaltung.domain.Bestellung;
import de.swe2.bestellverwaltung.service.BestellungDuplikatException;
import de.swe2.bestellverwaltung.service.BestellungValidationException;
import de.swe2.bestellverwaltung.service.Bestellverwaltung;
import de.swe2.kundenverwaltung.domain.AbstractKunde;
import de.swe2.kundenverwaltung.service.InvalidKundeIdException;
import de.swe2.kundenverwaltung.service.Kundenverwaltung;
import de.swe2.util.AbstractSwe2Exception;
import de.swe2.util.Log;
import de.swe2.util.MessagesMgdBean;
import de.swe2.util.MessagesMgdBean.MessagesType;
import de.swe2.util.NotFoundException;
import de.swe2.util.RequiredTx;


@Named("bv")
@SessionScoped
@RequiredTx
@Log
public class BestellverwaltungMgdBean implements Serializable {
	private static final long serialVersionUID = -2076163006686790460L;
	
	private static final String JSF_LIST_BESTELLUNGEN = "/bestellverwaltung/listBestellungen";
	private static final String JSF_SHOW_WARENKORB = "/bestellverwaltung/showWarenkorb";
	private static final String JSF_OK_BESTELLUNG = "/bestellverwaltung/ok";
	
	private static final String OK_CMD = "cmd";
	private static final String OK_CMD_CREATE = "create";
	
	private static final String FLASH_BESTELLUNG = "bestellung";
	private static final String FLASH_BESTELLUNGEN = "bestellungen";
	
	private static final String REQUEST_LADENHUETER = "ladenhueter";
	
	private static final String CLIENT_ID_KUNDEID = "findForm:kundeId";
	private static final String MSG_KEY_KUNDE_NOT_FOUND = "listBestellungen.kundeNotFound";
	private static final String MSG_KEY_BESTELLUNGEN_EMPTY = "listBestellungen.bestellungenEmpty";
	
	private static final String CLIENT_ID_BESTELL_BUTTON = "warenkorbForm:bestellButton";
	private static final String MSG_KEY_BESTELLUNG_DUPLIKAT = "showWarenkorb.bestellungDuplikat";
	
	private static final int ANZAHL_LADENHUETER = 5;
	
	@EJB
	private Bestellverwaltung bv;
	
	@EJB
	private Kundenverwaltung kv;
	
	@Inject
	private MessagesMgdBean messages;   // get/set notwendig fuer @ManagedProperty
	
	@Inject
	private AuthMgdBean auth;
	
	private final Warenkorb warenkorb = new Warenkorb();
	
	private Long kundeId;

	@Override
	public String toString() {
		return "BestellverwaltungMgdBean [warenkorb=" + warenkorb + ", kundeId=" + kundeId + "]";
	}

	/**
	 * Keine set-Methode erforderlich, da im Attribut bestellungen nur das vom Awk gelieferte
	 * Ergebnis gepuffert wird.
	 */

	public Warenkorb getWarenkorb() {
		return warenkorb;
	}

	/**
	 */
	public String addWarenkorb(Artikel artikel) {
		warenkorb.add(artikel);
		return JSF_SHOW_WARENKORB;
	}
	
	public Long getKundeId() {
		return kundeId;
	}

	public void setKundeId(Long kundeId) {
		this.kundeId = kundeId;
	}

	/**
	 */
	public String deleteFromWarenkorb(Bestellposition bestellposition) {
		warenkorb.getPositionen().remove(bestellposition);
		return null;
	}
	
	
	/**
	 */
	public String bestellen() {
		auth.login_j_security_check();
		
		if (warenkorb == null || warenkorb.getPositionen() == null || warenkorb.getPositionen().isEmpty()) {
			return null;
		}
		
		final ExternalContext ctx = FacesContext.getCurrentInstance().getExternalContext();
		
		// Den eingeloggten Kunden mit seinen Bestellungen ermitteln
		final String username = auth.getUsername();
		final Locale locale = ctx.getRequestLocale();
		AbstractKunde kunde;
		try {
			kunde = kv.findKundeMitBestellungenById(Long.valueOf(username), locale);
		}
		catch (InvalidKundeIdException e) {
			// kann nicht passieren, da dieser Kunde gerade eingeloggt ist
			return JSF_DEFAULT_ERROR;
		}
		catch (NotFoundException e) {
			// kann nicht passieren, da dieser Kunde gerade eingeloggt ist
			return JSF_DEFAULT_ERROR;
		}
		
		// Aus dem Warenkorb nur Positionen mit Anzahl > 0
		List<Bestellposition> positionen = new ArrayList<Bestellposition>(warenkorb.getPositionen().size());
		for (Bestellposition bp : warenkorb.getPositionen()) {
			if (bp.getAnzahl() > 0) {
				positionen.add(bp);
			}
		}
		
		// Warenkorb zuruecksetzen
		warenkorb.reset();
		
		// Neue Bestellung mit neuen Bestellpositionen erstellen
		Bestellung bestellung = new Bestellung();
		bestellung.setBestellpositionen(positionen);
		
		// Bestellung mit vorhandem Kunden verknuepfen: dessen Bestellungen muessen geladen sein
		try {
			bestellung = bv.createBestellung(bestellung, kunde, locale, true);
		}
		catch (AbstractSwe2Exception e) {
			final String outcome = bestellenErrorMsg(e);
			return outcome;
		}
		
		// Aufbereiten fuer ok.xhtml
		final Map<String, Object> requestParams = FacesContext.getCurrentInstance()
										          .getExternalContext()
										          .getRequestMap();
		requestParams.put(OK_CMD, OK_CMD_CREATE);
		final Flash flash = ctx.getFlash();
		flash.put(FLASH_BESTELLUNG, bestellung);

		return JSF_OK_BESTELLUNG;
	}
	
	/**
	 */
	private String bestellenErrorMsg(AbstractSwe2Exception e) {
		String outcome = null;
		
		if (e.getClass().equals(BestellungValidationException.class)) {
			// Validierungsfehler KOENNEN NICHT AUFTRETEN, da Attribute durch JSF validiert wurden
			// und in der Klasse Bestellung keine Validierungs-Methoden vorhanden sind
			outcome = JSF_DEFAULT_ERROR;
		}
		else if  (e.getClass().equals(BestellungDuplikatException.class)) {
			final BestellungDuplikatException orig = (BestellungDuplikatException) e;
			messages.createFacesMessage(MessagesType.BESTELLVERWALTUNG,
					                    MSG_KEY_BESTELLUNG_DUPLIKAT,
					                    CLIENT_ID_BESTELL_BUTTON,
					                    SEVERITY_ERROR,
					                    orig.getKundeId(),
					                    orig.getErzeugt());
		}
		
		return outcome;
	}


	/**
	 */
	public String findBestellungenByKundeId() {
		final Locale locale = FacesContext.getCurrentInstance().getExternalContext().getRequestLocale();
		AbstractKunde kunde = null;
		try {
			kunde = kv.findKundeMitBestellungenById(kundeId, locale);
		}
		catch (AbstractSwe2Exception e) {
			final String outcome = findBestellungenByKundeIdErrorMsg(e);
			return outcome;
		}
		
		// Kunde nicht vorhanden
		if (kunde == null) {
			final String outcome = findBestellungenByKundeIdErrorMsg(kundeId);
			return outcome;
		}

		final List<Bestellung> bestellungen = kunde.getBestellungen();

		if (bestellungen.isEmpty()) {
			messages.createFacesMessage(MessagesType.BESTELLVERWALTUNG,
					                    MSG_KEY_BESTELLUNGEN_EMPTY,
					                    CLIENT_ID_KUNDEID,
					                    SEVERITY_ERROR,
					                    kunde.getId());
			return null;
		}
		
		final Flash flash = FacesContext.getCurrentInstance().getExternalContext().getFlash();
		flash.put(FLASH_BESTELLUNGEN, bestellungen);
		
		return JSF_LIST_BESTELLUNGEN;
	}
	
	/**
	 */
	private String findBestellungenByKundeIdErrorMsg(AbstractSwe2Exception e) {
		String outcome = null;
		
		if (e.getClass().equals(InvalidKundeIdException.class)) {
			// Ungueltige Kunde-ID
			final InvalidKundeIdException orig = (InvalidKundeIdException) e;
			messages.createFacesMessage(orig.getViolations(), CLIENT_ID_KUNDEID, SEVERITY_ERROR);
		}
		
		return outcome;
	}

	private String findBestellungenByKundeIdErrorMsg(Long id) {
		messages.createFacesMessage(MessagesType.BESTELLVERWALTUNG,
				                    MSG_KEY_KUNDE_NOT_FOUND,
				                    CLIENT_ID_KUNDEID,
				                    SEVERITY_ERROR,
				                    id);
		return null;
	}
	
	public void loadLadenhueter() {
		List<Artikel> ladenhueter;
		try {
			ladenhueter = bv.ladenhueter(ANZAHL_LADENHUETER);
		}
		catch (NotFoundException e) {
			ladenhueter = Collections.emptyList();
		}
		final Map<String, Object> requestMap = FacesContext.getCurrentInstance().getExternalContext().getRequestMap();
		requestMap.put(REQUEST_LADENHUETER, ladenhueter);
	}
}
