package de.swe2.kundenverwaltung.ui;


import static de.swe2.util.WebConstants.JSF_DEFAULT_ERROR;
import static de.swe2.util.WebConstants.JSF_INDEX;
import static javax.faces.application.FacesMessage.SEVERITY_ERROR;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.ejb.EJB;
import javax.enterprise.context.SessionScoped;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.context.Flash;
import javax.faces.event.ValueChangeEvent;
import javax.inject.Inject;
import javax.inject.Named;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.swe2.admin.ui.AuthMgdBean;
import de.swe2.kundenverwaltung.domain.AbstractKunde;
import de.swe2.kundenverwaltung.domain.Adresse;
import de.swe2.kundenverwaltung.domain.Privatkunde;
import de.swe2.kundenverwaltung.domain.Privatkunde.Hobby;
import de.swe2.kundenverwaltung.service.InvalidKundeIdException;
import de.swe2.kundenverwaltung.service.InvalidNachnameException;
import de.swe2.kundenverwaltung.service.KundeDeleteBestellungException;
import de.swe2.kundenverwaltung.service.KundeDuplikatException;
import de.swe2.kundenverwaltung.service.KundeValidationException;
import de.swe2.kundenverwaltung.service.Kundenverwaltung;
import de.swe2.util.AbstractSwe2Exception;
import de.swe2.util.Log;
import de.swe2.util.MessagesMgdBean;
import de.swe2.util.RequiredTx;
import de.swe2.util.MessagesMgdBean.MessagesType;
import de.swe2.util.NotFoundException;

/**
 * Dialogsteuerung fuer die Kundenverwaltung
 */
@Named("kv")
@SessionScoped
@RequiredTx
@Log
public class KundenverwaltungMgdBean implements Serializable {
	private static final long serialVersionUID = -8817180909526894740L;
	
	private static final Logger LOGGER = LoggerFactory.getLogger(KundenverwaltungMgdBean.class);

	private static final String JSF_SHOW_KUNDE_DETAILS = "/kundenverwaltung/showKundeDetails";
	private static final String JSF_LIST_KUNDEN = "/kundenverwaltung/listKunden";
	private static final String JSF_UPDATE_PRIVATKUNDE = "/kundenverwaltung/updatePrivatkunde";
	private static final String JSF_UPDATE_FIRMENKUNDE = "/kundenverwaltung/updateFirmenkunde";
	private static final String JSF_OK_KUNDE = "/kundenverwaltung/ok";
	
	private static final String OK_CMD = "cmd";
	private static final String OK_CMD_CREATE = "create";
	private static final String OK_CMD_UPDATE = "update";
	private static final String OK_CMD_DELETE = "delete";
	private static final String OK_KUNDE_ID = "kundeId";
	
	private static final String FLASH_KUNDE = "kunde";
	private static final String FLASH_KUNDEN = "kunden";
	private static final String FLASH_NEUERKUNDE = "neuerKunde";
	
	private static final String REQUEST_KUNDE = "kunde";

	private static final String CLIENT_ID_KUNDEID = "findForm:kundeId";
	private static final String MSG_KEY_KUNDE_NOT_FOUND_BY_ID = "showKundeDetails.kundeNotFound";

	private static final String CLIENT_ID_NACHNAME = "findForm:nachname";
	private static final String MSG_KEY_KUNDE_NOT_FOUND_BY_NACHNAME = "listKunden.kundeNotFound";
	
	private static final String CLIENT_ID_CREATE_NACHNAME = "createKundeForm:nachname";
	private static final String MSG_KEY_CREATE_PRIVATKUNDE_DUPLIKAT = "createPrivatkunde.duplikat";
	private static final String CLIENT_ID_CREATE_PASSWORD = "createKundeForm:password";
	
	private static final String CLIENT_ID_UPDATE_PASSWORD = "updateKundeForm:password";
	private static final String CLIENT_ID_UPDATE_NACHNAME = "updateKundeForm:nachname";
	private static final String MSG_KEY_UPDATE_PRIVATKUNDE_DUPLIKAT = "updatePrivatkunde.duplikat";
	private static final String MSG_KEY_UPDATE_FIRMENKUNDE_DUPLIKAT = "updateFirmenkunde.duplikat";
	
	//private static final String CLIENT_ID_SELECT_DELETE_BUTTON_PREFIX = "selectForm:kundenTabelle:";
	//private static final String CLIENT_ID_SELECT_DELETE_BUTTON_SUFFIX = ":deleteButton";
	private static final String MSG_KEY_SELECT_DELETE_KUNDE_BESTELLUNG = "listKunden.deleteKundeBestellung";
	
	private static final String CLIENT_ID_DELETE_BUTTON = "ausgewaehlterKundeForm:deleteButton";
	private static final String MSG_KEY_DELETE_KUNDE_BESTELLUNG = "showKundeDetails.deleteKundeBestellung";
	
	@EJB
	private Kundenverwaltung kv;
	
	@Inject
	private MessagesMgdBean messages;
	
	@Inject
	private AuthMgdBean auth;   // get/set notwendig fuer @ManagedProperty

	private Long kundeId;
	private String nachname;
	private boolean geaendertKunde;    // fuer ValueChangeListener
	private Privatkunde neuerPrivatkunde;
	
	/**
	 */
	@PostConstruct
	private void initEmptyPrivatkunde() {
		neuerPrivatkunde = new Privatkunde();
		neuerPrivatkunde.setAdresse(new Adresse());
		neuerPrivatkunde.setHobbies(new ArrayList<Hobby>());
	}

	public Long getKundeId() {
		return kundeId;
	}

	public void setKundeId(Long kundeId) {
		this.kundeId = kundeId;
	}

	public String getNachname() {
		return nachname;
	}

	public void setNachname(String nachname) {
		this.nachname = nachname;
	}

	public Privatkunde getNeuerPrivatkunde() {
		return neuerPrivatkunde;
	}

	@Override
	public String toString() {
		return "KundenverwaltungMgdBean [kundeId=" + kundeId + ", nachname=" + nachname
		       + ", geaendertKunde=" + geaendertKunde + "]";
	}


	/**
	 * Action Methode, um einen Kunden zu gegebener ID zu suchen
	 * @return URL fuer Anzeige des gefundenen Kunden; sonst null
	 */
	public String findKundeById() {
		final Flash flash = FacesContext.getCurrentInstance().getExternalContext().getFlash();

		AbstractKunde kunde = null;
		final Locale locale = FacesContext.getCurrentInstance().getExternalContext().getRequestLocale();
		try {
			kunde = kv.findKundeById(kundeId, locale);
		}
		catch (AbstractSwe2Exception e) {
			final String outcome = findKundeByIdErrorMsg(e, kundeId);
			flash.remove(FLASH_KUNDE);			
			return outcome;
		}
		
		flash.put(FLASH_KUNDE, kunde);

		return JSF_SHOW_KUNDE_DETAILS;
	}
	
	/**
	 */
	private String findKundeByIdErrorMsg(AbstractSwe2Exception e, Long kundeId) {
		String outcome = null;
		
		if (e.getClass().equals(InvalidKundeIdException.class)) {
			final InvalidKundeIdException orig = (InvalidKundeIdException) e;
			messages.createFacesMessage(orig.getViolations(),
					                    CLIENT_ID_KUNDEID,
					                    SEVERITY_ERROR);
		}
		else if (e.getClass().equals(NotFoundException.class)) {
			messages.createFacesMessage(MessagesType.KUNDENVERWALTUNG,
					                    MSG_KEY_KUNDE_NOT_FOUND_BY_ID,
					                    CLIENT_ID_KUNDEID,
					                    SEVERITY_ERROR,
					                    kundeId);
		}
		
		return outcome;
	}
	
	/**
	 */
	public void loadKundeById() {
		final ExternalContext ctx = FacesContext.getCurrentInstance().getExternalContext();

		// Request-Parameter "kundeId" fuer ID des gesuchten Kunden
		final String idStr = ctx.getRequestParameterMap().get("kundeId");
		Long kundeId;
		try {
			kundeId = Long.valueOf(idStr);
		}
		catch (NumberFormatException e) {
			return;
		}
		
		// Suche durch den Anwendungskern
		AbstractKunde kunde = null;
		final Locale locale = ctx.getRequestLocale();
		try {
			kunde = kv.findKundeById(kundeId, locale);
		}
		catch (AbstractSwe2Exception e) {
			return;
		}
		
		final Map<String, Object> requestMap = ctx.getRequestMap();
		requestMap.put(REQUEST_KUNDE, kunde);
	}
	
	/**
	 * Action Methode, um einen Kunden zu gegebener ID zu suchen
	 * @return URL fuer Anzeige des gefundenen Kunden; sonst null
	 */
	public String findKundenByNachname() {
		List<AbstractKunde> kunden = null;
		if (nachname == null || nachname.isEmpty()) {
			try {
				kunden = kv.findAllKunden();
			}
			catch (NotFoundException e) {
				kunden = Collections.emptyList();
			}
			return JSF_LIST_KUNDEN;
		}

		final Flash flash = FacesContext.getCurrentInstance().getExternalContext().getFlash();

		// Suche nach dem plausiblen Nachnamen
		final Locale locale = FacesContext.getCurrentInstance().getExternalContext().getRequestLocale();
		try {
			kunden = kv.findKundenByNachname(nachname, locale);
		}
		catch (AbstractSwe2Exception e) {
			final String outcome = findKundenByNachnameErrorMsg(e, nachname);
			flash.remove(FLASH_KUNDEN);
			return outcome;
		}
		
		flash.put(FLASH_KUNDEN, kunden);
		return JSF_LIST_KUNDEN;
	}
	
	/**
	 */
	private String findKundenByNachnameErrorMsg(AbstractSwe2Exception e, String nachname) {
		String outcome = null;
		
		if (e.getClass().equals(InvalidNachnameException.class)) {
			final InvalidNachnameException orig = (InvalidNachnameException) e;
			messages.createFacesMessage(orig.getViolations(),
					                    CLIENT_ID_NACHNAME,
					                    SEVERITY_ERROR);
		}
		if (e.getClass().equals(NotFoundException.class)) {
			messages.createFacesMessage(MessagesType.KUNDENVERWALTUNG,
					                    MSG_KEY_KUNDE_NOT_FOUND_BY_NACHNAME,
					                    CLIENT_ID_NACHNAME,
					                    SEVERITY_ERROR,
					                    nachname);
		}
		
		return outcome;
	}
	
	/**
	 * Verwendung als ValueChangeListener bei updatePrivatkunde.xhtml und updateFirmenkunde.xhtml
	 */
	public void geaendert(ValueChangeEvent e) {
		if (geaendertKunde) {
			return;
		}
		
		if (e.getOldValue() == null) {
			if (e.getNewValue() != null) {
				geaendertKunde = true;
			}
			return;
		}

		if (!e.getOldValue().equals(e.getNewValue())) {
			geaendertKunde = true;				
		}
	}
	
	/**
	 */
	public String details(AbstractKunde kunde) {
		if (kunde == null) {
			return null;
		}
		
		kundeId = kunde.getId();
		final Flash flash = FacesContext.getCurrentInstance().getExternalContext().getFlash();
		flash.put(FLASH_KUNDE, kunde);
		return JSF_SHOW_KUNDE_DETAILS;
	}

	/**
	 */
	public String update() {
		auth.login_j_security_check();
		
		if (!geaendertKunde) {
			return JSF_INDEX;
		}
		
		final Flash flash = FacesContext.getCurrentInstance().getExternalContext().getFlash();
		AbstractKunde kunde = (AbstractKunde) flash.get(FLASH_KUNDE);
		if (kunde != null) {
			LOGGER.trace(kunde.toString());
		}

		final Locale locale = FacesContext.getCurrentInstance().getExternalContext().getRequestLocale();
		try {
			kunde = kv.updateKunde(kunde, locale, true, false);
		}
		catch (AbstractSwe2Exception e) {
			final String outcome = updateErrorMsg(e, kunde.getClass());
			return outcome;
		}
		
		if (kunde == null) {
			return JSF_DEFAULT_ERROR;
		}
		
		// ValueChangeListener zuruecksetzen
		geaendertKunde = false;
		
		// Aufbereitung fuer ok.xhtml
		final Map<String, Object> requestParams = FacesContext.getCurrentInstance()
										          .getExternalContext()
										          .getRequestMap();
		requestParams.put(OK_CMD, OK_CMD_UPDATE);
		return JSF_OK_KUNDE;
	}
	
	/**
	 */
	private String updateErrorMsg(AbstractSwe2Exception e, Class<? extends AbstractKunde> kundeClass) {
		String outcome = null;
		
		if (e.getClass().equals(KundeValidationException.class)) {
			// Ungueltiges Password: Attribute wurden bereits von JSF validiert
			final KundeValidationException orig = (KundeValidationException) e;
			messages.createFacesMessage(orig.getViolations(),
					                    CLIENT_ID_UPDATE_PASSWORD,
					                    SEVERITY_ERROR);
		}
		else if (e.getClass().equals(KundeDuplikatException.class)) {
			if (kundeClass.equals(Privatkunde.class)) {
				messages.createFacesMessage(MessagesType.KUNDENVERWALTUNG,
						                    MSG_KEY_UPDATE_PRIVATKUNDE_DUPLIKAT,
						                    CLIENT_ID_UPDATE_NACHNAME,
						                    SEVERITY_ERROR);
			}
			else {
				messages.createFacesMessage(MessagesType.KUNDENVERWALTUNG,
						                    MSG_KEY_UPDATE_FIRMENKUNDE_DUPLIKAT,
						                    CLIENT_ID_UPDATE_NACHNAME,
						                    SEVERITY_ERROR);
			}
		}
		else if (e.getClass().equals(NotFoundException.class)) {
			// Kann nicht passieren, da der Kunde vor der Modifikation geladen/gesucht werden musste
			outcome = JSF_DEFAULT_ERROR;
		}
		
		return outcome;
	}
	
	/**
	 * Action Methode, um einen zuvor gesuchten Kunden zu l&ouml;schen
	 * @return URL fuer Startseite im Erfolgsfall, sonst wieder die gleiche Seite
	 */
	public String deleteAngezeigtenKunden() {
		auth.login_j_security_check();
		
		final ExternalContext ctx = FacesContext.getCurrentInstance().getExternalContext();
		final Flash flash = ctx.getFlash();
		final AbstractKunde kunde = (AbstractKunde) flash.get(FLASH_KUNDE);
		if (kunde == null) {
			return null;
		}
		
		LOGGER.trace(kunde.toString());
		
		final Locale locale = ctx.getRequestLocale();
		try {
			kv.deleteKunde(kunde, locale);
		}
		catch (KundeDeleteBestellungException e) {
			flash.put(FLASH_KUNDE, kunde);
			messages.createFacesMessage(MessagesType.KUNDENVERWALTUNG,
					                    MSG_KEY_DELETE_KUNDE_BESTELLUNG,
					                    CLIENT_ID_DELETE_BUTTON,
					                    SEVERITY_ERROR,
					                    e.getKundeId(),
					                    e.getAnzahlBestellungen());
			return null;
		}
		catch (InvalidKundeIdException e) {
			flash.put(FLASH_KUNDE, kunde);
			messages.createFacesMessage(MessagesType.KUNDENVERWALTUNG,
					                    MSG_KEY_KUNDE_NOT_FOUND_BY_ID,
					                    CLIENT_ID_DELETE_BUTTON,
					                    SEVERITY_ERROR,
					                    e.getId());
			return null;
		}
		
		// Aufbereitung fuer ok.xhtml
		final Map<String, Object> requestParams = FacesContext.getCurrentInstance()
										          .getExternalContext()
										          .getRequestMap();
		requestParams.put(OK_CMD, OK_CMD_DELETE);
		requestParams.put(OK_KUNDE_ID, kunde.getId());
		return JSF_OK_KUNDE;
	}
	
	/**
	 */
	public String selectForUpdate(AbstractKunde kunde) {
		final Flash flash = FacesContext.getCurrentInstance().getExternalContext().getFlash();
		flash.put(FLASH_KUNDE, kunde);
		
		if (AbstractKunde.PRIVATKUNDE.equals(kunde.getArt())) {
			return JSF_UPDATE_PRIVATKUNDE;
		}
		else {
			return JSF_UPDATE_FIRMENKUNDE;
		}
	}

	/**
	 */
	@SuppressWarnings("unchecked")    // wegen Flash.get()
	public String delete(AbstractKunde kunde) {
		auth.login_j_security_check();

		final ExternalContext ctx = FacesContext.getCurrentInstance().getExternalContext();
		final Flash flash = ctx.getFlash();
		final List<AbstractKunde> kunden = (List<AbstractKunde>) flash.get(FLASH_KUNDEN);
		if (kunden == null) {
			LOGGER.error("KundenverwaltungMgdBean.delete(): flash.get(\"kunden\") == null");
			return JSF_INDEX;
		}
		
		final Locale locale = ctx.getRequestLocale();
		try {
			kv.deleteKunde(kunde, locale);
		}
		catch (KundeDeleteBestellungException e) {
			//final int zeilennr = tabelleListKunden.getRowIndex();
			messages.createFacesMessage(MessagesType.KUNDENVERWALTUNG,
					                    MSG_KEY_SELECT_DELETE_KUNDE_BESTELLUNG,
			                            //CLIENT_ID_SELECT_DELETE_BUTTON_PREFIX + zeilennr + CLIENT_ID_SELECT_DELETE_BUTTON_SUFFIX,
			                            null,
			                            SEVERITY_ERROR,
			                            e.getKundeId(),
			                            e.getAnzahlBestellungen());
			
			flash.put(FLASH_KUNDEN, kunden);
			return null;
		}
		catch (InvalidKundeIdException e) {
			messages.createFacesMessage(MessagesType.KUNDENVERWALTUNG,
					                    MSG_KEY_KUNDE_NOT_FOUND_BY_ID,
                                        null,
                                        SEVERITY_ERROR,
                                        e.getId());

			flash.put(FLASH_KUNDEN, kunden);
			return null;
		}

		kunden.remove(kunde);
		flash.put(FLASH_KUNDEN, kunden);
		return null;
	}
	
	/**
	 */
	public String createPrivatkunde() {
		auth.login_j_security_check();
		
		final ExternalContext ctx = FacesContext.getCurrentInstance().getExternalContext();
		final Locale locale = ctx.getRequestLocale();
		
		try {
			neuerPrivatkunde = (Privatkunde) kv.createKunde(neuerPrivatkunde, locale, true);
		}
		catch (AbstractSwe2Exception e) {
			final String outcome = createPrivatkundeErrorMsg(e);
			return outcome;
		}
		
		// Aufbereitung fuer ok.xhtml
		final Map<String, Object> requestParams = ctx.getRequestMap();
		requestParams.put(OK_CMD, OK_CMD_CREATE);
		final Flash flash = ctx.getFlash();
		flash.put(FLASH_NEUERKUNDE, neuerPrivatkunde);
		
		// Aufbereitung fuer showKundeDetails.xhtml
		kundeId = neuerPrivatkunde.getId();

		initEmptyPrivatkunde();
		return JSF_OK_KUNDE;

		// Redirect:
		// - ok.xhtml steht in der URL des Webbrowsers
		// - Wie ein zusaetzlicher Http-Request, d.h. Flash geht dann jedoch verloren
		//return JSF_OK_KUNDE + "?faces-redirect=true";

	}

	
	/**
	 */
	private String createPrivatkundeErrorMsg(AbstractSwe2Exception e) {
		String outcome = null;
		
		if (e.getClass().equals(KundeValidationException.class)) {
			// Ungueltiges Password: Attribute wurden bereits von JSF validiert
			final KundeValidationException orig = (KundeValidationException) e;
			messages.createFacesMessage(orig.getViolations(),
					                    CLIENT_ID_CREATE_PASSWORD,
					                    SEVERITY_ERROR);
		}
		else if (e.getClass().equals(KundeDuplikatException.class)) {
			messages.createFacesMessage(MessagesType.KUNDENVERWALTUNG,
					                    MSG_KEY_CREATE_PRIVATKUNDE_DUPLIKAT,
					                    CLIENT_ID_CREATE_NACHNAME,
					                    SEVERITY_ERROR);
		}
		else if (e.getClass().equals(NotFoundException.class)) {
			// kann nur bei einem Programmierfehler passieren
			outcome = JSF_DEFAULT_ERROR;
		}
		
		return outcome;
	}
	

	/**
	 */
	public Date getAktuellesDatum() {
		final Date datum = new Date();
		return datum;
	}
}
