package de.swe2.util;

import static javax.transaction.Status.STATUS_ACTIVE;

import java.io.Serializable;

import javax.annotation.Resource;
import javax.interceptor.AroundInvoke;
import javax.interceptor.Interceptor;
import javax.interceptor.InvocationContext;
import javax.transaction.UserTransaction;

@Interceptor
@RequiredTx
public class RequiredTxInterceptor implements Serializable {
	private static final long serialVersionUID = 4012853821451097541L;
	
	@Resource
	private UserTransaction tx;

	@AroundInvoke
    public Object intercept(InvocationContext ctx) throws Throwable {
		boolean startedTransaction = false;
		if (tx.getStatus() != STATUS_ACTIVE) {
			// Transaktion nur starten, falls es keine get-/set-Methode ist
			if (!ctx.getMethod().getName().startsWith("get") && !ctx.getMethod().getName().startsWith("set")) {
				tx.begin();
				startedTransaction = true;
			}
		}
		
		Object result = null;
		try {
			result = ctx.proceed();
			if (startedTransaction && tx.getStatus() == STATUS_ACTIVE) {
				tx.commit();
			}
		}
		catch (Throwable t) {
			if (startedTransaction && tx.getStatus() == STATUS_ACTIVE) {
				tx.rollback();
			}
			throw t;
		}
		
		return result;
	}
}
